; ##############################################################################
; ##############################################################################
; ##############################################################################
; WDTCSR  Watchdog Timer Control Register				
; WDIF WDIE WDP3 WDCE WDE WDP2 WDP1 WDP0				
; WDP3 WDP2 WDP1 WDP0 Time								
;	0 	0 	0 	0 	2K 	(2048) 	cycles 16 ms			
;	0 	0 	0 	1 	4K 	(4096) 	cycles 32 ms			
;	0 	0 	1 	0 	8K 	(8192) 	cycles 64 ms			
;	0 	0 	1 	1 	16K (16384) cycles 0.125 s			
;	0 	1 	0 	0 	32K (32768) cycles 0.25 s			
; 	0 	1 	0 	1 	64K (65536) cycles 0.5 s			
; *	0 	1 	1 	0 	128K (131072)cycles 1.0 s			
;	0 	1 	1 	1 	256K (262144)cycles 2.0 s			
;	1 	0 	0 	0 	512K (524288)cycles 4.0 s			
;	1 	0 	0 	1 	1024K(1048576)cycles 8.0 s			
; Wenn WDIE=1 dann wird zuerst die WDR-ISR aufgerufen 	
; und dann das System neu gestartet						
WDR_INITIALISIERUNG:
	WDR
	LDS TEMP, WDTCSR
	ori TEMP, (1<<WDCE) | (1<<WDE)
	STS WDTCSR, TEMP
	ldi temp, (1<<WDE)| (1<<WDIE)|(0<<WDP3)|(1<<WDP2)|(1<<WDP1)|(0<<WDP0)
	STS WDTCSR, TEMP
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; Diese Routine wird bei WDR-Zeitberschreitung aufgerufen, hier knnte man		
; sich z.B. den Fehler anzeigen lasen, oder DATEN in den EEPROM schreiben		
WDT:
	WDR
	CLI											; INTERRUPTS sperren	
	CLI											; INTERRUPTS sperren	
	call LCD_CLEAR
	call LCD_W
	call LCD_D
	call LCD_R
	call LCD_ZEILE2
	call TEXT_ERROR
WDT_s:
	sbi (LED_GRUEN_PORT),(LED_GRUEN_PIN_NR)		; GRN aus	
	sbi (LED_ROT_PORT)  ,(LED_ROT_PIN_NR)		; ROT  aus	
	call wait_100ms
	cbi (LED_ROT_PORT),LED_ROT_PIN_NR			; ROT  an	
	call wait_100ms
	call TASTEN_ABFRAGE
	rjmp WDT_s
; ##############################################################################
; ##############################################################################
; ##############################################################################
SRAM_CLEAR:
	ldi ZL, LOW (SRAM_START)
	ldi ZH, HIGH(SRAM_START)
	ldi temp,0
SRAM_CLEAR_w:
	ST z+,temp
	cpi ZH, HIGH(RAMEND-2)	; Vergleich?		
	brlo SRAM_CLEAR_w		; kleiner => SPRUNG	
	cpi ZL, LOW(RAMEND-2)	; Vergleich?		
	brlo SRAM_CLEAR_w		; kleiner => SPRUNG	
	breq SRAM_CLEAR_w		; gleich  => SPRUNG	
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
STARTWERTE:
	; NULL						
	clr NULL
	; EINS						
	ldi temp,1
	mov EINS,temp
	; SRAM CHECK				
	ldi temp,'S'
	STS(adr_SRAM_CHECK),temp
	; SLEEP						
	ldi temp,(SLEEPWERT)				; Sleep		
	STS(adr_FLOPPY_SLEEP),temp
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; PORT CLEAR																	
PORT_CLEAR:
	ldi temp,0
	out DDRA,temp 
	out DDRB,temp 
	out DDRC,temp
	out DDRD,temp 
	ldi temp,0xFF
	out PORTA,temp
	out PORTB,temp
	out PORTC,temp
	out PORTD,temp
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
PROGRAMM_CHECK:
	; NULL		
	tst NULL
	brne PROGRAMM_CHECK_ERROR
	; EINS		
	mov temp,EINS
	cpi temp,1
	brne PROGRAMM_CHECK_ERROR
	; SRAM CHECK
	LDS temp,(adr_SRAM_CHECK)
	cpi temp,'S'
	brne PROGRAMM_CHECK_ERROR
	ret
;-------------------------------------------------------------------------------
PROGRAMM_CHECK_ERROR:
	CLI											; INTERRUPTS sperren	
	call LCD_CLEAR
	call LCD_C
	call LCD_H
	call LCD_E
	call LCD_C
	call LCD_K
	call LCD_ZEILE2
	call TEXT_ERROR
PROGRAMM_CHECK_ERROR_s:
	sbi (LED_GRUEN_PORT),(LED_GRUEN_PIN_NR)		; GRN aus	
	sbi (LED_ROT_PORT)  ,(LED_ROT_PIN_NR)		; ROT  aus	
	call wait_100ms
	cbi (LED_ROT_PORT),(LED_ROT_PIN_NR)			; ROT  an	
	call wait_100ms
	rjmp PROGRAMM_CHECK_ERROR_s
; ##############################################################################
; ##############################################################################
; ##############################################################################
LED_INITIALISIERUNG:
	sbi (LED_ROT_DDR)  ,(LED_ROT_PIN_NR)		; ROT		
	sbi (LED_GRUEN_DDR),(LED_GRUEN_PIN_NR)		; GRN		
;-------------------------------------------------------------------------------
	cbi (LED_GRUEN_PORT),(LED_GRUEN_PIN_NR)		; GRN an	
	call WAIT_100ms
	cbi (LED_ROT_PORT)  ,(LED_ROT_PIN_NR)		; ROT  an	
	call WAIT_100ms
;-------------------------------------------------------------------------------
	sbi (LED_GRUEN_PORT),(LED_GRUEN_PIN_NR)		; GRN aus	
	sbi (LED_ROT_PORT)  ,(LED_ROT_PIN_NR)		; ROT  aus	
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
PROGRAMM_STOP:
	WDR
	rjmp PROGRAMM_STOP
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ##############################################################################
SEKTOR_NULL:
	clr SEKTOR0
	clr SEKTOR1
	clr SEKTOR2
	clr SEKTOR3
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
SEKTOR_PLUS_EINS:
	add SEKTOR0,EINS
	adc SEKTOR1,NULL
	adc SEKTOR2,NULL
	adc SEKTOR3,NULL
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
SEKTOR_MINUS_EINS:
	sub SEKTOR0,EINS
	sbc SEKTOR1,NULL
	sbc SEKTOR2,NULL
	sbc SEKTOR3,NULL
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
SEKTOR_MAL_2:
	LSL SEKTOR0					; x2	
	ROL SEKTOR1
	ROL SEKTOR2
	ROL SEKTOR3
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
SEKTOR_DURCH_2:
	LSR SEKTOR3					; :2	
	ROR SEKTOR2
	ROR SEKTOR1
	ROR SEKTOR0
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TEMP1_4_NACH_SEKTOR:
	MOV SEKTOR0,temp1
	MOV SEKTOR1,temp2
	MOV SEKTOR2,temp3
	MOV SEKTOR3,temp4
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
SEKTOR_NACH_TEMP1_4:
	MOV temp1,SEKTOR0
	MOV temp2,SEKTOR1
	MOV temp3,SEKTOR2
	MOV temp4,SEKTOR3
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TEMP1_2_3_4_PLUS_EINS:
	add temp1,EINS						; +1		
	adc temp2,NULL
	adc temp3,NULL
	adc temp4,NULL
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TEMP1_2_3_4_MINUS_EINS:
	sub temp1,EINS						; -1		
	sbc temp2,NULL
	sbc temp3,NULL
	sbc temp4,NULL
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TEMP1_2_3_4_DURCH_2:
	LSR TEMP4					; :2	
	ROR TEMP3
	ROR TEMP2
	ROR TEMP1
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TEMP1_2_3_4_MAL_2:
	LSL TEMP1					; x2	
	ROL TEMP2
	ROL TEMP3
	ROL TEMP4
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TEMP1_2_3_4_MAL_512:
	mov temp4,temp3			; 8xlinks		
	mov temp3,temp2
	mov temp2,temp1
	clr temp1				; LOW			

	LSL temp1				; LOW 1xlinks	
	ROL temp2
	ROL temp3
	ROL temp4				; HIGH			
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TEMP1_2_3_4_DURCH_512:
	mov temp1,temp2			; 8xlinks		
	mov temp2,temp3
	mov temp3,temp4
	clr temp4
	LSR TEMP4				; :2	
	ROR TEMP3
	ROR TEMP2
	ROR TEMP1
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TEMP1_2_3_4_DURCH_1000:
	ldi temp5,LOW (1000)
	ldi temp6,HIGH(1000)
	rjmp DIVISION_32_16
; ##############################################################################
; ##############################################################################
; ##############################################################################
; Version: 14.11.2014																	
;																						
; 	INPUT:	TEMP1 / TEMP2 	1.Faktor													
;			TEMP3 / TEMP4	2.Faktor													
; 																						
; 	OUTPUT: TEMP1...4	Produkt															
; 																						
;	HILFSREGISTER: 	temp5+6+7+8 (werden gesichert)										
; 																						
MULTIPLIKATION_16_16_32:
	push temp5
	push temp6
	push temp7
	push temp8

	clr	r2
	mul	temp4,temp2		; ah * bh
	mov	temp7,r0
	mov	temp8,r1
	mul	temp3,temp1		; al * bl
	mov	temp5,r0
	mov	temp6,r1
	mul	temp4,temp1		; ah * bl
	add	temp6,r0
	adc	temp7,r1
	adc	temp8,r2
	mul	temp2,temp3		; bh * al
	add	temp6,r0
	adc	temp7,r1
	adc	temp8,r2


	mov temp1,temp5
	mov temp2,temp6
	mov temp3,temp7
	mov temp4,temp8

	pop temp8
	pop temp7
	pop temp6
	pop temp5

ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; Version: 11/2014																
;																				
;	Division 32 bit-Zahl durch 16 bit Zahl										
;																				
;	input/Output 	temp1 (LOW)	(0...4.294.967.295)								
;	input/Output 	temp2														
;	input/Output 	temp3														
;	input/Output 	temp4 (HIGH)												
;																				
;	input	temp5	Divisor	(LOW)	 											
;	input	temp6	Divisor	(HIGH)	 											
;																				
; Register:	R0+R1	Hilfsregister 												
;			temp	Hilfsregister 												
;																				
DIVISION_32_16:
	ldi     temp,32    ;32* schieben	
    clr     R0          ;Rest =0		
    clr     R1
DIVISION_32_16_1:    
	lsl     temp1       ;Ein Bit aus dem Dividend ...   
    rol     temp2       ;ins Carry schieben
    rol     temp3
    rol     temp4
    rol     R0          ;Carry in den Rest schieben
    rol     R1
    brcs    DIVISION_32_16_3       ;Bei berlauf von Rest, Sprung
    cp      R0,temp5       					;Solange Bits in temp1 (Rest) schieben bis >= divisor
    cpc     R1,temp6
    brcs    DIVISION_32_16_2    	;Bei Rest<Divisor, die eingeschobene 0 in temp1 stehen lassen
DIVISION_32_16_3:
    inc     temp1       ;Ansonsten Bit0 setzen und den Divisor vom Rest abziehen
    sub     R0,temp5
    sbc     R1,temp6
DIVISION_32_16_2:
   	dec     temp       ;Das ganze 32 mal
    brne    DIVISION_32_16_1
ret 
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ##############################################################################
; inp: 		temp8 		(BIT-START)	
; inp; 		temp9		(BIT_ENDE)	
; ZL/ZH:	SRAM					
; OUT: 		temp1..5				
; Hilfsregister: R0, R1				
FUNKTION_SRAM_SELECT_Z_8_9:
	clr temp1
	clr temp2
	clr temp3
	clr temp4
	clr temp5
	ldi temp,0b10000000
	mov R1,temp			; Hilfsregister	
	inc temp9
	sub temp9,temp8		; Schleife		
FUNK_SRAM_s:
	mov temp,temp8		; (BIT-NR)		
	andi temp,0b11111000
	LSR temp			; rechts ohne C	
	LSR temp
	LSR temp
	push ZL				; Z sichern		
	push ZH
	sub ZL,temp			; subtrahieren	
	sbc ZH,NULL
	LD R0,Z				; load			
	pop ZH				; Z wiederherstellen
	pop ZL
; ------------------------------------------------------------------------------
	mov temp,temp8		; (BIT-NR)			
	andi temp,0b00000111
	cpi temp,0
	breq FUNK_SRAM0
	cpi temp,1
	breq FUNK_SRAM1
	cpi temp,2
	breq FUNK_SRAM2
	cpi temp,3
	breq FUNK_SRAM3
	cpi temp,4
	breq FUNK_SRAM4
	cpi temp,5
	breq FUNK_SRAM5
	cpi temp,6
	breq FUNK_SRAM6
	rjmp FUNK_SRAM7
; ------------------------------------------------------------------------------
FUNK_SRAM0:
	CLC				; Clear C	
	SBRC R0,0
	SEC 			; Set C		
	rjmp FUNK_SRAM_w
FUNK_SRAM1:
	CLC				; Clear C	
	SBRC R0,1
	SEC 			; Set C		
	rjmp FUNK_SRAM_w
FUNK_SRAM2:
	CLC				; Clear C	
	SBRC R0,2
	SEC 			; Set C		
	rjmp FUNK_SRAM_w
FUNK_SRAM3:
	CLC				; Clear C	
	SBRC R0,3
	SEC 			; Set C		
	rjmp FUNK_SRAM_w
FUNK_SRAM4:
	CLC				; Clear C	
	SBRC R0,4
	SEC 			; Set C		
	rjmp FUNK_SRAM_w
FUNK_SRAM5:
	CLC				; Clear C	
	SBRC R0,5
	SEC 			; Set C		
	rjmp FUNK_SRAM_w
FUNK_SRAM6:
	CLC				; Clear C	
	SBRC R0,6
	SEC 			; Set C		
	rjmp FUNK_SRAM_w
FUNK_SRAM7:
	CLC				; Clear C	
	SBRC R0,7
	SEC 			; Set C		
	rjmp FUNK_SRAM_w
FUNK_SRAM_w:
	rcall FUNK_SRAM_RECHTS_C	; rechts ber C
	inc temp8		; (BIT-NR)	
	dec temp9		; Schleife	
	breq FUNK_SRAM_w2
	rjmp FUNK_SRAM_s
; ------------------------------------------------------------------------------
FUNK_SRAM_w2:
	SBRC temp1,0	
	rjmp FUNK_SRAM_w3
	CLC				; Clear C	
	rcall FUNK_SRAM_RECHTS_C	; rechts ber C
	rjmp FUNK_SRAM_w2
; ------------------------------------------------------------------------------
FUNK_SRAM_w3:
	CLC				; Clear C	
	rcall FUNK_SRAM_RECHTS_C	; rechts ber C
	ret
; ------------------------------------------------------------------------------
FUNK_SRAM_RECHTS_C:
	ROR R1			; rechts ber C
	ROR temp5
	ROR temp4
	ROR temp3
	ROR temp2
	ROR temp1
ret
