; ##############################################################################
; ##############################################################################
; ##############################################################################
; TIMER0 8-BIT-TIMER															
;																				
; CS02 CS01 CS00																
;	0 	0	0 	No clock source (Timer/Counter stopped)							
;  *0 	0 	1 	clkI/O/(No prescaling)											
; 	0 	1 	0 	clkI/O/8 (From prescaler)										
;	0 	1 	1 	clkI/O/64 (From prescaler)										
;	1 	0 	0 	clkI/O/256 (From prescaler)										
;	1 	0 	1 	clkI/O/1024 (From prescaler)									
;	1 	1 	0 	External clock source on T0 pin. Clock on falling edge.			
;	1 	1 	1 	External clock source on T0 pin. Clock on rising edge.			
;																				
; Mode WGM2 WGM1 WGM0															
; 0    0  	0 	0 	Normal 0xFF Immediate MAX 									
; 1    0 	0 	1 	PWM, Phase Correct 0xFF TOP BOTTOM							
; 2    0 	1 	0 	CTC OCRA Immediate MAX										
; 3    0 	1 	1 	Fast PWM 0xFF BOTTOM MAX									
; 4    1 	0 	0 	Reserved   												
; 5    1 	0 	1 	PWM, Phase Correct OCRA TOP BOTTOM							
; 6    1 	1 	0 	Reserved   												
; 7    1 	1 	1 	Fast PWM OCRA BOTTOM TOP									
;																				
; Frequenz an PIN PB3 (OC0A)													
;																				
TIMER0_INITIALISIERUNG:
	; TCCR0A  Timer/Counter Control Register A		
	ldi temp, (0<<COM0A1)| (0<<COM0A0)|(0<<COM0B1)|(0<<COM0B0)|(0<<WGM01)|(0<<WGM00)
	OUT TCCR0A,temp
	; TCCR0B  Timer/Counter Control Register B		
	ldi temp, (0<<FOC0A)| (0<<FOC0B)|(0<<WGM02)|(0<<CS02)|(0<<CS01)|(1<<CS00)
	OUT TCCR0B,temp
	; TIMSK0  Timer/Counter Interrupt Mask Register
	ldi temp, (0<<OCIE0B)| (0<<OCIE0A)|(0<<TOIE0)
	STS TIMSK0,temp
	; OCR0A  Output Compare Register A				
	OUT OCR0A,NULL
	; STARTWERT										
	OUT TCNT0,NULL
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; 16-BIT-TIMER																	
;																				
; CS32 CS31 CS30																
;   0 	0 	0 	No clock source (Timer/Counter stopped).						
;   0 	0 	1 	clkI/1 (No prescaling)											
; 	0 	1 	0 	clkI/8 (From prescaler)											
; *	0 	1 	1 	clkI/64 (From prescaler)										
; 	1 	0 	0 	clkI/256 (From prescaler)										
; 	1 	0 	1 	clkI/1024 (From prescaler)										
; 	1 	1 	0 	External clock source on Tn pin. Clock on falling edge.			
; 	1 	1 	1 	External clock source on Tn pin. Clock on rising edge.			
;																				
;Uhr-Timer																		
TIMER3_INITIALISIERUNG:
	; TCCR3A  Timer/Counter 3 Control Register A			
	ldi temp, (0<<COM3A1)| (0<<COM3A0)|(0<<COM3B1)|(0<<COM3B0)|(0<<WGM31)|(0<<WGM30)
	STS TCCR3A,temp
	; TCCR3B  Timer/Counter 3 Control Register B			
	ldi temp, (0<<ICNC3)| (0<<ICES3)|(0<<WGM33)|(0<<WGM32)|(0<<CS32)|(1<<CS31)|(1<<CS30)
	STS TCCR3B,temp
	; TCCR3C  Timer/Counter 3 Control Register C			
	ldi temp, (0<<FOC3A)|(0<<FOC3B)
	STS TCCR3C,temp
	; TIMSK3  Timer/Counter3 Interrupt Mask Register		
	ldi temp, (0<<ICIE3)| (0<<OCIE3B)|(0<<OCIE3A)|(1<<TOIE3)
	STS TIMSK3,temp
	; OCR3AH and OCR3AL  Output Compare Register 3 A		
	ldi temp,HIGH(TIM3WERT)
	STS OCR3AH,temp
	ldi temp,LOW (TIM3WERT)
	STS OCR3AL,temp
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TIM3_OVF:
	push TEMP		    ; Sichern von "TEMP" im Stack			
	in	 TEMP,SREG		; Einlesen des SREG 					
	push TEMP	    	; Schreiben von  SREG  im Stack (KOPIE)	
	ldi temp,LOW (TIM3WERT)
	STS TCNT3L,temp
	ldi temp,HIGH (TIM3WERT)
	STS TCNT3H,temp

	rcall UHR


	pop	TEMP			; LESEN von SREG vom STACK (KOPIE)		
	out	SREG,TEMP		; Wiederherstellen von SREG				
	pop	TEMP			; Wiederherstellen von "TEMP"
reti
; ##############################################################################
; ##############################################################################
; ##############################################################################
UHR:
	LDS temp,(adr_ss_VT)
	inc temp
	cpi temp,10
	brsh UHR_ss
	STS(adr_ss_VT),temp
	ret
;-------------------------------------------------------------------------------
UHR_ss:
	LDS temp,(adr_FLOPPY_SLEEP)
	tst temp
	breq UHR_ss_w
	dec temp
	STS(adr_FLOPPY_SLEEP),temp
UHR_ss_w:
	STS(adr_ss_VT),NULL
	LDS temp,(adr_ss)
	inc temp
	cpi temp,60
	brsh UHR_mm
	STS(adr_ss),temp
	ret
;-------------------------------------------------------------------------------
UHR_mm:
	STS(adr_ss),NULL

	LDS temp,(adr_mm)
	inc temp
	cpi temp,60
	brsh UHR_hh
	STS(adr_mm),temp
	ret
;-------------------------------------------------------------------------------
UHR_hh:
	STS(adr_mm),NULL

	LDS temp,(adr_hh)
	inc temp
	cpi temp,24
	brsh UHR_TAG
	STS(adr_hh),temp
	ret
;-------------------------------------------------------------------------------
UHR_TAG:
	STS(adr_hh),NULL
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
UHR_NULL:
	STS(adr_SS_VT),NULL
	STS(adr_ss),NULL
	STS(adr_mm),NULL
	STS(adr_hh),NULL
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
