; TWI-SCL										
.equ TWI_SCL_PIN_NR			= 0
.equ TWI_SCL_PORT			= PORTC
; TWI-SDA										
.equ TWI_SDA_PIN_NR			= 1
.equ TWI_SDA_PORT			= PORTC
; #############################################################################
; #############################################################################
; #############################################################################
TWI_INITIALISIERUNG:
; TWI MASTER EINSTELLUNGEN						
.equ TWI_BIT_RATE							= 255		; 18	 Bit-Rate							
.equ TWI_PRESCALER							= 64		; 1		 Vorteiler: 0=1 1=4 2=16 3=64	

	; SCL(Hz) = CPU(Hz) / (16 + 2 * TWBR * 4^TWPS)	
	; Beispiel:										
	; 100.000 = 16.000.000 / (16 + 2 * 18 * 4^1)	

	; TWI TWBR 				Bit RATE Register	
	ldi temp, TWI_BIT_RATE
	STS TWBR, temp
	; TWI TWSR 				STATUS-Register		
	ldi temp, TWI_PRESCALER
	STS TWSR, temp
	; TWI TWAR (nur SLAVE) 	Adress-Register		
	clr temp
	STS TWAR,temp
	; TWI TWCR 				Control-REGISTER	
	ldi temp, 1<<TWINT|1<<TWEA|0<<TWSTA|0<<TWSTO|0<<TWWC|1<<TWEN|0<<TWIE
	STS TWCR,temp
	; PULL-UP ein								
	sbi (TWI_SCL_PORT),(TWI_SCL_PIN_NR)
	sbi (TWI_SDA_PORT),(TWI_SDA_PIN_NR)

ret
; #############################################################################
; #############################################################################
; #############################################################################
TWI_AUS:
	ldi temp, 1<<TWINT|0<<TWEA|0<<TWSTA|0<<TWSTO|0<<TWWC|0<<TWEN|0<<TWIE
	STS TWCR,temp
ret
; #############################################################################
; #############################################################################
; #############################################################################
;																			   
; START-ROUTINE	(MASTER TRANSMITTER)										   
;																			   
; INPUT:	temp1 <= ADRESSE SLAVE											   
; OUTPUT: 	FEHLER_CODE														   
;																			   
TWI_MT_START_TEMP1:
	WDR
	clr ERROR
	; START								
	ldi temp, (1<<TWINT)|(1<<TWSTA)|(1<<TWEN)
	STS TWCR, temp	
	; warten, bis TWINT wieder = 1		
	rcall TWI_TWINT_wait									
	; Check TWSR						
	LDS temp,TWSR													
	andi temp, 0xF8
	cpi temp,  0x08							; 0x08 START erfolgreich
	breq TWI_MT_START_w
	cpi temp,  0x10							; 0x10 (repeated START)
	breq TWI_MT_START_w
	rjmp TWI_MT_START_ERROR
;-------------------------------------------------------------------------------
TWI_MT_START_w:
	; Load SLA + "W"  (W=0) 			
	LSL  temp1								; 1 x links
	STS TWDR, temp1
	; senden - Clear TWINT 				
	ldi temp, (1<<TWINT) | (1<<TWEN)							
	STS TWCR, temp
	; warten, bis TWINT wieder = 1		
	rcall TWI_TWINT_wait									
	; Check TWSR						
	LDS temp,TWSR												
	andi temp, 0xF8
	cpi temp,  0x18							; 0x18
	brne TWI_MT_START_ERROR
	ret
;-------------------------------------------------------------------------------
TWI_MT_START_ERROR:
	mov ERROR, EINS
ret
; #############################################################################
; #############################################################################
; #############################################################################
; 																			   
; Ein BYTE zum SLAVE zu senden										   	   	   
; 																			   
; INPUT:	temp1 <= DATEN													   
; OUT: 		FEHLER_CODE		   												   
; 																			   
TWI_MT_DATA_TEMP1:
	tst ERROR								; Fehler =0 ?	
	breq TWI_MT_DATA_w						; ja			
	ret										; zurck		
;-------------------------------------------------------------------------------
TWI_MT_DATA_w:
	; 	Load DATA						
	STS TWDR, temp1							
	; senden - Clear TWINT 				
	ldi temp, (1<<TWINT) | (1<<TWEN)							
	STS TWCR, temp
	; warten, bis TWINT wieder = 1		
	rcall TWI_TWINT_wait									
	; Check TWSR						
	LDS temp,TWSR
	andi temp, 0xF8
	cpi temp, 0x28	; 0x28
	brne TWI_MT_DATA_ERROR
	ret
;-------------------------------------------------------------------------------
TWI_MT_DATA_ERROR:
	mov ERROR, EINS
ret
; #############################################################################
; #############################################################################
; #############################################################################
;																			   
; INPUT:	temp1 <= ADRESSE SLAVE											   
; OUT: 		FEHLER_CODE		   												   
;																			   
TWI_MR_START_TEMP1:
	WDR
	clr ERROR
;-------------------------------------------------------------------------------
	; SRAM lschen						
	ldi ZL,LOW (adr_TWI_BEREICH_READ)
	ldi ZH,HIGH(adr_TWI_BEREICH_READ)
	clr temp9
TWI_MR_START_s:
	ST Z+,NULL
	dec temp9
	brne TWI_MR_START_s
;-------------------------------------------------------------------------------
	; START 											
	ldi temp, (1<<TWINT | 1<<TWEA | 1<<TWSTA | 1<<TWEN )
	STS TWCR, temp	
	; warten, bis TWINT wieder = 1						
	rcall TWI_TWINT_wait
	; Check TWSR 										
	LDS temp,TWSR		
	andi temp, 0xF8
	cpi temp,  0x08 								; 0x08 START erfolgreich
	breq TWI_MR_START_w
	cpi temp,  0x10									; 0x10 (repeated START)
	breq TWI_MR_START_w
	rjmp TWI_MR_START_ERROR
;-------------------------------------------------------------------------------
TWI_MR_START_w:
	; Load SLA +"R" (R=1)								
	lsl  temp1	; 1 x links
	ori temp1, 0b00000001 
	STS TWDR, temp1
	; senden - Clear TWINT 								
	ldi temp, (1<<TWINT | 1<<TWEA  | 1<<TWEN )
	STS TWCR, temp
	; warten, bis TWINT wieder = 1						
	rcall TWI_TWINT_wait
	; Check TWSR										
	LDS temp,TWSR
	andi temp, 0xF8
	cpi temp, 0x40 	;SLA+R gesendet; ACK empfangen
	brne TWI_MR_START_ERROR
	ret
;-------------------------------------------------------------------------------
TWI_MR_START_ERROR:
	mov ERROR, EINS
ret
; #############################################################################
; #############################################################################
; #############################################################################
;																				
; Achtung, diese Routine verwenden, wenn 										
; mehrere! Bytes empfangen werden sollen										
;																				
; OUTPUT: temp1																	
; OUTPUT: FEHLER_CODE		   													
;																				
TWI_MR_DATA_TEMP1:
	tst ERROR
	brne TWI_MR_DATA_ERROR
;-------------------------------------------------------------------------------
	; senden "ACK" (TWEA=1) fr mehrere Daten-Bytes 	
	ldi temp, (1<<TWINT | 1<<TWEA  | 1<<TWEN )
	STS TWCR, temp
	; TWINT abfragen									
	rcall TWI_TWINT_wait
	; Check TWSR										
	LDS temp,TWSR
	andi temp, 0xF8
	cpi temp, 0x50		; Data Byte empfangen- ACK zurckgesendet
	brne TWI_MR_DATA_ERROR
	LDS temp1, TWDR		; DATEN einlesen												
	ret
;-------------------------------------------------------------------------------
TWI_MR_DATA_ERROR:
	mov ERROR, EINS	
	clr temp1
ret
; #############################################################################
; #############################################################################
; #############################################################################
;																				
; Achtung, diese Routine verwenden, wenn 										
; nur ein! Zeichen empfangen werden soll, oder wenn								
; das letzte! Zeichen empfangen werden soll										
;																				
; OUTPUT: temp1																	
;																				
TWI_MR_DATA_LAST_TEMP1:
	tst ERROR
	brne TWI_MR_DATA_LAST_ERROR
;-------------------------------------------------------------------------------
	; senden "NOT ACK" (TWEA=0) fr letztes bzw fr ein Daten-Byte
	ldi temp, (1<<TWINT | 0<<TWEA  | 1<<TWEN )
	STS TWCR, temp
	; TWINT abfragen									
	rcall TWI_TWINT_wait
	; Check TWSR										
	LDS temp,TWSR
	andi temp, 0xF8
	cpi  temp, 0x58	; letztes Data Byte empfangen-NOT ACK zurckgesendet
	brne TWI_MR_DATA_LAST_ERROR
	LDS temp1, TWDR	; DATEN einlesen		
	ret
;-------------------------------------------------------------------------------
TWI_MR_DATA_LAST_ERROR:
	mov ERROR, EINS	
	clr temp1
ret
; #############################################################################
; #############################################################################
; #############################################################################
; 																			   
; STOP- Routine																   
; 																			   
TWI_STOP:
	ldi temp, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)|0<<TWIE	; STOP								
	STS TWCR, temp	
	; kurze Pause
	call wait_1ms
ret
; #############################################################################
; #############################################################################
; #############################################################################
; 																			   
; ERROR- Routine															   
; 																			   
TWI_ERROR:
	ldi temp, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)	; TWI STOP				
	STS TWCR, temp
	call wait_100ms	
	ldi temp, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)	; TWI STOP				
	STS TWCR, temp
	ldi temp, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)	; TWI STOP				
	STS TWCR, temp
	ldi temp, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)	; TWI STOP				
	STS TWCR, temp
	ldi temp, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)	; TWI STOP				
	STS TWCR, temp
	ldi temp, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)	; TWI STOP				
	STS TWCR, temp
	ldi temp, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)	; TWI STOP				
	STS TWCR, temp
	ldi temp, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)	; TWI STOP				
	STS TWCR, temp
	ldi temp, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)	; TWI STOP				
	STS TWCR, temp
ret
; #############################################################################
; #############################################################################
; #############################################################################
; 																			   
; warten, bis das TWINT Flag gesetzt ist									   
; 																			   
TWI_TWINT_wait:
	LDS temp, TWCR
	sbrs temp, TWINT	; nchsten Befehl berspringen, wenn TWINT FLAG  gesetzt ist
	rjmp TWI_TWINT_wait	
ret
; #############################################################################
; #############################################################################
; #############################################################################
; #############################################################################
; #############################################################################
; #############################################################################

